//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Global.h"
#include "NymRepWizs.h"
#include "NymAnonToDlgs.h"
#include "NymRemEncChainDlgs.h"
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TNymAnonToDlg *NymAnonToDlg;
int const mNone = 0;
int const mExt = 1;
//---------------------------------------------------------------------------

bool __fastcall EditAnonToDirectives(TForm *Owner, AnsiString &AnonTo,
   AnsiString &RemixTo, AnsiString &EncryptTo, AnsiString &RemailTo,
   AnsiString &TestTo, AnsiString &Password,
   const AnsiString &Caps, bool Last)
{
   NymAnonToDlg = new TNymAnonToDlg(Owner);

   NymAnonToDlg->AnonTo->Text = AnonTo;
   NymAnonToDlg->RemixTo->Text = RemixTo;
   NymAnonToDlg->EncryptTo->Text = EncryptTo;
   NymAnonToDlg->RemailTo->Text = RemailTo;
   NymAnonToDlg->TestTo->Text = TestTo;
   NymAnonToDlg->Password->Text = Password;
   NymAnonToDlg->Caps = Caps;
   NymAnonToDlg->Last = Last;

   NymAnonToDlg->ShowModal();

   bool ok = NymAnonToDlg->ModalResult == mrOk;
   if (ok)
   {
      AnonTo = NymAnonToDlg->AnonTo->Text;
      RemixTo = NymAnonToDlg->RemixTo->Text;
      EncryptTo = NymAnonToDlg->EncryptTo->Text;
      RemailTo = NymAnonToDlg->RemailTo->Text;
      TestTo = NymAnonToDlg->TestTo->Text;
      Password = NymAnonToDlg->Password->Text;
   }

   delete NymAnonToDlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TNymAnonToDlg::TNymAnonToDlg(TComponent* Owner)
        : TForm(Owner)
{
   CenterDialog(this, Owner);

   AnonToBtn->Left = AnonToBox->Left;
   RemixToBtn->Left = RemixToBox->Left;
   EncryptToBtn->Left = EncryptToBox->Left;
   RemailToBtn->Left = RemailToBox->Left;
}
//---------------------------------------------------------------------------

bool __fastcall TNymAnonToDlg::HasCap(const AnsiString &Cap)
{
   return Caps.Pos(" " + Cap + " ") > 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::EnableControl(TEdit *Control, bool Enable)
{
   Control->Color = Enable ? clWindow : clBtnFace;
   Control->Enabled = Enable;
   //if (Enable)
   //   ActiveControl = Control;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymAnonToDlg::GetAnonToAddress()
{
   AnsiString address, tmp = AnonTo->Text;
   if (!tmp.IsEmpty())
      return tmp;

   if (tmp.IsEmpty())
      tmp = RemixTo->Text;
   if (tmp.IsEmpty())
      tmp = EncryptTo->Text;
   if (tmp.IsEmpty())
      tmp = RemailTo->Text;

   for (int i = tmp.Length(); i > 0; i--)
   {
      if (tmp[i] == ',' || tmp[i] == ' ')
         break;

      address = AnsiString(tmp[i]) + address;
   }

   return address;
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::FormShow(TObject *Sender)
{
   bool remix = HasCap("remix");
   if (Last || !remix)
   {
      RemixToChainBtn->Enabled = false;
      RemixToBtn->Enabled = false;
      RemixToBox->Enabled = false;
      RemixTo->Enabled = false;
      RemixTo->Color = clBtnFace;
   }

   bool repgp = HasCap("repgp");
   if (Last || !repgp)
   {
      EncryptToChainBtn->Enabled = false;
      EncryptToBtn->Enabled = false;
      EncryptToBox->Enabled = false;
      EncryptTo->Enabled = false;
      EncryptTo->Color = clBtnFace;
   }

   mode = HasCap("ext") ? mExt : mNone;

   if (mode == mNone)
   {
      AnonToBtn->Visible = true;
      AnonToBox->Visible = false;

      RemixToBtn->Visible = true;
      RemixToBox->Visible = false;

      EncryptToBtn->Visible = true;
      EncryptToBox->Visible = false;

      RemailToBtn->Visible = true;
      RemailToBox->Visible = false;

      RemixToChainBtn->Visible = false;
      RemixTo->Width = AnonTo->Width;
      EncryptToChainBtn->Visible = false;
      EncryptTo->Width = AnonTo->Width;

      if (!AnonTo->Text.IsEmpty())
         AnonToBtn->Checked = true;
      else if (!RemixTo->Text.IsEmpty())
         RemixToBtn->Checked = true;
      else if (!EncryptTo->Text.IsEmpty())
         EncryptToBtn->Checked = true;
      else if (!RemailTo->Text.IsEmpty())
         RemailToBtn->Checked = true;
   }
   else
   {
      AnonToBtn->Visible = false;
      AnonToBox->Visible = true;

      RemixToBtn->Visible = false;
      RemixToBox->Visible = true;

      EncryptToBtn->Visible = false;
      EncryptToBox->Visible = true;

      RemailToBtn->Visible = false;
      RemailToBox->Visible = true;

      if (!AnonTo->Text.IsEmpty())
         AnonToBox->Checked = true;
      if (HasCap("remix") && !RemixTo->Text.IsEmpty())
         RemixToBox->Checked = true;
      if (HasCap("repgp") && !EncryptTo->Text.IsEmpty())
         EncryptToBox->Checked = true;
      if (!RemailTo->Text.IsEmpty())
         RemailToBox->Checked = true;

      if (!AnonToBox->Checked)
         EnableControl(AnonTo, false);
      if (!RemixToBox->Checked)
      {
         EnableControl(RemixTo, false);
         RemixToChainBtn->Enabled = false;
      }
      if (!EncryptToBox->Checked)
      {
         EnableControl(EncryptTo, false);
         EncryptToChainBtn->Enabled = false;
      }
      if (!RemailToBox->Checked)
         EnableControl(RemailTo, false);
   }

   AnsiString addr = GetAnonToAddress();
   if (AnonTo->Text.IsEmpty())
      AnonTo->Text = addr;

   if (!Last && remix && RemixTo->Text.IsEmpty())
      RemixTo->Text = addr;
   if (!Last && repgp && EncryptTo->Text.IsEmpty())
      EncryptTo->Text = addr;

   if (RemailTo->Text.IsEmpty())
      RemailTo->Text = addr;

   RemixTo->Modified = false;
   EncryptTo->Modified = false;
   TestTo->Modified = false;
   Password->Modified = false;
}
//---------------------------------------------------------------------------
void __fastcall TNymAnonToDlg::AnonToBtnClick(TObject *Sender)
{
   EnableControl(AnonTo, true);
   EnableControl(RemixTo, false);
   EnableControl(EncryptTo, false);
   EnableControl(RemailTo, false);
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::RemixToBtnClick(TObject *Sender)
{
   EnableControl(AnonTo, false);
   EnableControl(RemixTo, true);
   EnableControl(EncryptTo, false);
   EnableControl(RemailTo, false);
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::EncryptToBtnClick(TObject *Sender)
{
   EnableControl(AnonTo, false);
   EnableControl(RemixTo, false);
   EnableControl(EncryptTo, true);
   EnableControl(RemailTo, false);
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::RemailToBtnClick(TObject *Sender)
{
   EnableControl(AnonTo, false);
   EnableControl(RemixTo, false);
   EnableControl(EncryptTo, false);
   EnableControl(RemailTo, true);
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::AnonToBoxClick(TObject *Sender)
{
   EnableControl(AnonTo, AnonToBox->Checked);
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::RemixToBoxClick(TObject *Sender)
{
   bool checked = RemixToBox->Checked;
   RemixToChainBtn->Enabled = checked;
   EnableControl(RemixTo, checked);
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::EncryptToBoxClick(TObject *Sender)
{
   bool checked = EncryptToBox->Checked;
   EncryptToChainBtn->Enabled = checked;
   EnableControl(EncryptTo, checked);
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::RemailToBoxClick(TObject *Sender)
{
   EnableControl(RemailTo, RemailToBox->Checked);
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::TestToExit(TObject *Sender)
{
   if (!TestTo->Text.IsEmpty())
   {
      EnableControl(Password, true);
      Label1->Enabled = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::TestToChange(TObject *Sender)
{
   bool enable = !TestTo->Text.IsEmpty();
   Password->Enabled = enable;
   Password->Color = enable ? clWindow : clBtnFace;
   Label1->Enabled = Password->Enabled;
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::PasswordEnter(TObject *Sender)
{
   if (Password->Text.IsEmpty())
      Password->Text = GetEncryptKey();
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::RemixToChainBtnClick(TObject *Sender)
{
   AnsiString remixto = RemixTo->Text;
   if (EditRemixEncryptChain(this, "Select Remix-To Remailer Chain", remixto))
   {
      RemixTo->Text = remixto;
      RemixTo->Modified = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::EncryptToChainBtnClick(TObject *Sender)
{
   AnsiString encryptto = EncryptTo->Text;
   if (EditRemixEncryptChain(this, "Select Encrypt-To Remailer Chain", encryptto))
   {
      EncryptTo->Text = encryptto;
      EncryptTo->Modified = true;
   }
}
//---------------------------------------------------------------------------

bool __fastcall TNymAnonToDlg::ValidAddress(AnsiString Address)
{
   if (Address.IsEmpty() || (mode == mExt && Address.LowerCase() == "me"))
      return true;

   int at = 0; for (int i = 0; i < Address.Length(); i++)
      if (Address[i] == '@')
         at++;

   if (at > 1)
   {
      MsgDlg(this, "Your Test-To address is not a valid email address!   ", "Error", MB_OK);
      return false;
   }

   at = Address.Pos("@");
   int pt = Address.SubString(at + 1, 256).Pos(".");
   if (at == 0 || pt == 0)
   {
      MsgDlg(this, "Your Test-To address is not a valid email address!   ", "Error", MB_OK);
      return false;
   }

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TNymAnonToDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrCancel)
   {
      if (RemixTo->Modified || EncryptTo->Modified || TestTo->Modified || Password->Modified)
      {
         switch(MsgDlg(this, "Your delivery options are modified. Save your changes?   ", "Confirm",
            MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
            case IDYES: ModalResult = mrOk;
         }
      }
   }

   if (ModalResult == mrOk)
   {
      if (!ValidAddress(TestTo->Text))
      {
         CanClose = false;
         return;
      }
   }
}
//---------------------------------------------------------------------------
